#include <stdio.h>
#include <unistd.h>
#include "DspLib.h"
#include "WavePlay.h"
#include "WaveHeader.h"

/*
** ouvre un fichier WAV et se prepare a le jouer
*/
t_wave_file wave_open_for_use (wav_path)
     char	*wav_path;
{
  t_wave_file	temp_wav;

  temp_wav = wave_read_header(wav_path);
  if (temp_wav.file == -1)
    return (temp_wav);
  if (!is_wave(temp_wav.header))
    {
#ifdef TEXT_MSG
      printf ("erreur, le fichier n'est pas au format WAVE\n");
#endif
      temp_wav.file = -1;
      return (temp_wav);
    }
  /* On se positionne apres le header */
  lseek(temp_wav.file, sizeof(t_wave_header), 0); 
  temp_wav.processed_bytes = 0;

  return (temp_wav);
}

/*
** joue des 'portions' de BUFFER_SIZE octets du WAV
*/
int wave_update_play_buffer(temp_wav)
     t_wave_file	*temp_wav;
{
  int	n;
  char	buffer[BUFFER_SIZE];

  n = read(temp_wav->file, &buffer, BUFFER_SIZE);
  temp_wav->processed_bytes += n;
  if (temp_wav->processed_bytes > temp_wav->header.data_size) 
    n -= (temp_wav->processed_bytes - temp_wav->header.data_size);
  write(temp_wav->dsp_file, &buffer, n);
  
  return (n == BUFFER_SIZE);
}

/*
** joue le fichier WAV donner par 'wav_path'
*/
int wave_play(wav_path)
     char	*wav_path;
{
  t_dsp*	my_dsp;
  t_wave_file	wav_file;

  wav_file = wave_open_for_use(wav_path); 
  if (wav_file.file == -1)
    return (-1);
#ifdef TEXT_MSG
    printf("%d canaux / %d bits / %liHz\n", wav_file.header.chanels,wav_file.header.bits_per_sample, wav_file.header.sample_rate);
#endif

    my_dsp = open_dsp_for_play(DSP_DEVICE, wav_file.header.bits_per_sample, wav_file.header.chanels, wav_file.header.sample_rate,"");

    wav_file.dsp_file = my_dsp->file;

    dsp_lock(my_dsp);

    while (wave_update_play_buffer(&wav_file)) 
      {}; /* Mise a jour du buffer */
    close_dsp(my_dsp);
    wave_close(wav_file);
    return (0);
}

















