/*
** Load_WAV.c for Load_WAV (Plug in for Gmurf) in ~/Gmurf/Load_WAV/
** 
** Made by 
** Login   <ahes@ker-ys.epita.fr>
** 
** Started on  Wed Feb 17 23:14:38 1999 
** Last update Sun Mar 14 01:48:02 1999 
*/
#include <errno.h>
#include "Save_WAV.h"

/*
** save_the_wav: create the target file with the header and the wave and close it
*/
int save_the_wav (temp, wav_header, file_name)
     t_tempgmurf	*temp;
     t_wave_header	*wav_header;
     char		*file_name;
{ 
  int		fd;
  char		buf[BUFFER_SIZE];
  off_t		cpt;
  off_t		size;

  fd = open(file_name, O_CREAT | O_TRUNC | O_APPEND | O_RDWR, 0644); /* create and open the file */
  write(fd, wav_header, sizeof(t_wave_header));			     /* write the header in the file */
  cpt = size = 0;
  while (cpt < temp->temp_size) 
    {
      size = cpt + BUFFER_SIZE < temp->temp_size ? BUFFER_SIZE : temp->temp_size - cpt;
      read(temp->file, buf, size);
      write(fd, buf, size);
      cpt += size;
    };
  close(fd);
  return (0);
}

/*
** print_error: print an error
*/
void	print_error(msg)
char	*msg;
{
int	child_result;
PID_T   pid;

#ifdef TEXT_MSG
  printf("%s\n   errno = %d\n", msg, errno);
#endif
  pid = fork();
  if (pid == 0)
    {
      if(execlp(ERROR_PRINTER, ERROR_PRINTER, msg) == -1)
      {
#ifdef TEXT_MSG
	  printf("erreur lors de l'execution de execve, errno = %d\n", errno);
#endif
	}
    }
  else
    waitpid(pid, &child_result, WUNTRACED);
  exit(-1);
}

/*
** make_WAV_header: create the header for the target wave file
*/
t_wave_header	*make_WAV_header(tempgmurf, rate)
t_tempgmurf	*tempgmurf;
int		rate;
{
  t_wave_header	*wav_header;

  wav_header = (t_wave_header *) malloc(sizeof(t_wave_header));
  strcpy (wav_header->riff_str, "RIFF");
  wav_header->size	        = tempgmurf->temp_size + sizeof(t_wave_header) - 8;
  strcpy(wav_header->wave_fmt_str, "WAVEfmt ");		     
  wav_header->smt_size		= 16;
  wav_header->format_tag	= 1;
  wav_header->chanels		= 2;
  wav_header->sample_rate	= rate;
  wav_header->avg_byte_per_sec	= rate * 4;
  wav_header->block_align	= 4;
  wav_header->bits_per_sample	= 2 * 8;
  strcpy(wav_header->data_str, "data");
  wav_header->data_size		= tempgmurf->temp_size; 

  return (wav_header);
}


int main (argc, argv)
int	argc;
char	*argv[];
{
  int		fd;
  int		rate;
  t_tempgmurf	*tempgmurf;
  t_wave_header	*wav_header;

  tempgmurf = (t_tempgmurf *) malloc(sizeof(t_tempgmurf));	/* malloc the tempgmurf */
  if (argc != 4)
    print_error(ARGS_ERROR);
  rate = atoi(argv[2]);						/* get the rate */
  if ((fd = open(argv[1], O_RDWR)) == -1)			/* open the swap file */
    print_error(SWAP_OPEN_ERROR);				
  if (read(fd, &(*tempgmurf), sizeof(t_tempgmurf)) == -1)	/* read the swapfile */
    print_error(SWAP_READ_ERROR);	
  close(fd);							/* close the swapfile */
  wav_header = make_WAV_header(tempgmurf, rate);		/* fill the wav_header struct */
  if ((tempgmurf->file = open(tempgmurf->tempname, O_RDONLY)) == -1)	/* open the tempfile */
    print_error(BAD_OPEN);
  if (lseek(tempgmurf->file, 0, SEEK_SET) == -1)
    print_error(BAD_SEEK);
  if (save_the_wav(tempgmurf, wav_header, argv[3]) != 0)	/* save the WAV file */
    print_error(WAV_OPEN_ERROR);
  close(tempgmurf->file);					/* close the tempfile */
  if (unlink(argv[1]) == -1)					/* del the swapfile */
    print_error(SWAP_DEL_ERROR);
  free(tempgmurf);
  free(wav_header);
  return(1);
}
